package org.example;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.util.UriComponentsBuilder;

@RestController
@RequestMapping("user")
public class ArticleController {
    @Autowired
    private IArticleService articleService;

    @Autowired
    private ArticleRepository articleRepository;

    //Fetches all articles
    @GetMapping(value= "articles")
    public ResponseEntity<List<ArticleInfo>> getAllArticles() {
        List<ArticleInfo> responseArticleList = new ArrayList<>();
        List<Article> articleList = articleService.getAllArticles();
        for (int i = 0; i < articleList.size(); i++) {
            ArticleInfo ob = new ArticleInfo();
            BeanUtils.copyProperties(articleList.get(i), ob);
            responseArticleList.add(ob);
        }
        return new ResponseEntity<List<ArticleInfo>>(responseArticleList, HttpStatus.OK);
    }

    //Creates a new article
    @PostMapping(value= "article")
    public ResponseEntity<Void> addArticle(@RequestBody ArticleInfo articleInfo, UriComponentsBuilder builder) {
        Article article = new Article();
        BeanUtils.copyProperties(articleInfo, article);
        articleService.addArticle(article);
        HttpHeaders headers = new HttpHeaders();
        headers.setLocation(builder.path("/article/{id}").buildAndExpand(article.getArticleId()).toUri());
        return new ResponseEntity<Void>(headers, HttpStatus.CREATED);
    }
    @GetMapping("/getOne")
    public Optional<Article> getOne(Long id) {
        return articleRepository.findById(id);
    }
    // 127.0.0.1:8080/user/getOne?id=1

    /**
     * 新增数据.
     * @param article 新增Test对象
     * @return
     */
    @PostMapping("/insert")
    public String insert(Article article) {
        System.out.println(article);
        articleRepository.save(article);
        return "success";
    }

    /**
     * 根据ID删除数据.
     * @param id ID
     * @return
     */
    @GetMapping("/delete")
    public String delete(Long id) {
        articleRepository.deleteById(id);
        return "success";
    }

    /**
     * 修改test数据.
     * @param article Test对象
     * @return
     */
    @PostMapping("/update")
    public String update(Article article) {
        articleRepository.save(article);
        return "success";
    }
}
