package org.example;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jms.annotation.JmsListener;
import org.springframework.jms.core.JmsMessagingTemplate;
import org.springframework.stereotype.Component;

import javax.jms.Queue;
import javax.jms.Topic;
import java.io.Serializable;
import java.util.Date;

@Component
public class JmsComponent {

    @Autowired
    private JmsMessagingTemplate jmsMessagingTemplate;

    @Autowired
    private Queue queue;
    @Autowired
    private Topic topic;

    /**
     * 发送消息
     */
    public void send(Message message){
        jmsMessagingTemplate.convertAndSend(this.queue,message.toString());
    }

    /**
     * 接收消息
     */
    @JmsListener(destination= "${queueName}")    //用这个注解去监听 监听的队列
    public void receive(String messageString){
        System.out.println("receive:"+messageString);
    }


    /**
     * 广播发送消息
     */
    public void sendForTopic(Message message){
        jmsMessagingTemplate.convertAndSend(this.topic,message.toString());
    }

    /**
     * 接收消息
     */
    @JmsListener(destination= "${topicName}",containerFactory = "queueListenerFactory")
    public void receiveForTopic1(String messageString){
        System.out.println("receive1:"+messageString);
    }

    /**
     * 接收消息
     */
    @JmsListener(destination= "${topicName}",containerFactory = "topicListenerFactory")
    public void receiveForTopic2(String messageString){
        System.out.println("receive2:"+messageString);
    }

}



